/* LED_PWM16

   LED fényerő 16 bites PWM vezérlése potméterrel, exponenciális
   jelleggörbével. A vezérlőjel a D10 kivezetésen jelenik meg.
   LED szalag esetén egy FET-tel oldjuk meg a teljesítmény illesztést.

   A 16 bites PWM Timer1 felhasználásával valósítható meg a D9 (OCR1A)
   illetve a D10 (OCR1B) kivezetéseken. Ebben a programban csak D10
   vezérlését implementáltuk.

   Módosítási lehetőségek:
   - Invertált PWM-hez a TCCR1A tegiszter COM1B0 bitje 1 legyen.
   - A D9 kiemenet használatához a TCCR1A tegiszter COM1A1 bitje 1 legyen,
     a COM1A0 bit szerepe pedig 0: nem invertált, 1: invertált kimenet
*/

const uint16_t MAX_PWM_VALUE = 65535;

void setup() {
  pinMode(10, OUTPUT);
  TCCR1A = bit(COM1B1) |               // Non-inv PWM
           bit(WGM11);                 // Mode 14: Fast PWM, TOP=ICR1
  TCCR1B = bit(WGM13) | bit(WGM12) |
           bit(CS10);                  // Prescaler 1
  ICR1 = MAX_PWM_VALUE;                // TOP counter value
}

void loop() {
  uint32_t sum = 0;
  for (uint16_t i = 0; i < 256; i++) {
    sum += analogRead(A1);
  }
  uint16_t val = sum/256;
  OCR1B = pow(val, 1.6002234458);
  delay(20);
}
