/* MF_breathingLED_16
 *  
 * "Lélegző" LED 16 bites PWM vezérléssel, exponenciális 
 * jelleggörbével. A program a Multifunkciós kártya D4 LED-jét
 * vezérli, azaz a LED a D10 kimenetre van kötve, s a katódot
 * vezéreljük.
 * 
 * A 16 bites PWM Timer1 felhasználásával valósítható meg a D9 (OCR1A)
 * illetve a D10 (OCR1B) kivezetéseken. Ebben a programban csak D10
 * vezérlését implementáltuk, s a katódvezérlés miatt invertált kimenetű
 * PWM jelet állítunk elő.
 * 
 * Módosítási lehetőségek:
 * - Nem invertált PWM-hez a TCCR1A tegiszter COM1B0 bitje 0 legyen.
 * - A D9 kiemenet használatához a TCCR1A tegiszter COM1A1 bitje 1 legyen,
 *   a COM1A0 bit szereőe pedig 0: nem invertált, 1: invertált kimenet 
 *   
 * A setup_7seg() függvény csak arra szolgál, hogy a multifunkciós kártya 
 * hétszegmenses kijelzőjét inicializálja és lekapcsolja.
 */

const uint16_t MAX_PWM_VALUE = 65535U;

void setup() {
  setup_7seg();                        // Multifunkciós kártya inicializálás
  pinMode(10, OUTPUT);
  TCCR1A = bit(COM1B0) | bit(COM1B1) | // Inverting PWM (katódvezérléshez)
           bit(WGM11);                 // Mode 14: Fast PWM, TOP=ICR1
  TCCR1B = bit(WGM13) | bit(WGM12) |
           bit(CS10);                  // Prescaler 1
  ICR1 = MAX_PWM_VALUE;                // TOP counter value
}

void loop() {
  for (uint16_t i = 0; i < 1023; i++) {
    OCR1B = pow(i, 1.6002234458);
    delay(2);
  }
  for (uint16_t i = 0; i < 1023; i++) {
    OCR1B = pow(1023 - i, 1.6002234458);
    delay(2);
  }
}

/**************************************************
 * A Multifunkciós kártya 7-szegmens kijelzőjének * 
 * inicializálása és elsötétítése                 *
 **************************************************/
#define LATCH_DIO 4
#define CLK_DIO   7
#define DATA_DIO  8
void setup_7seg(void) {
  /* Vezérlő lábak konfigurálása */
  pinMode(LATCH_DIO, OUTPUT);
  pinMode(CLK_DIO, OUTPUT);
  pinMode(DATA_DIO, OUTPUT);
  digitalWrite(LATCH_DIO, LOW);
  shiftOut(DATA_DIO, CLK_DIO, MSBFIRST, 0xFF);
  shiftOut(DATA_DIO, CLK_DIO, MSBFIRST, 0x00 );
  digitalWrite(LATCH_DIO, HIGH);
}
