/* led_pwm
 * Egy LED fényerejét változtatjuk egy potméter segítségével
 * az analogRead() és az analogWrite() függvény használatával.
 * 
 * Kapcsolás:
 * - Kössük egy potméter csúszkáját az A4 bemenetre (két végét
 *   pedig GND és +5V közé)!
 * - Kössük egy LED katódját a D10 kimenetre, anódját pedig  
 *   egy 470 ohmos áramkorlátozó ellenálláson keresztül a 
 *   tápfeszültségre (+5V)!
 *                   ___________
 *                  |           |    LED   470R
 *         |--------| A4    D10 |----|<|--/\/\/\--- +5V
 * +5V -/\/\/\- GND |           |       
 *       10K        |  Arduino  |
 */
 
/********************************************************************************
 * A dim_curve korrekciós táblázat az emberi látás nemlineáris voltát kompenzálja,
 * a fényerőszabályozás így természetesebbnek tűnik.
 * Az alábbi értékeket közelítőleg az y = round(pow( 2.0, [x+64]/40.0) - 1)
 * képlettel írhatjuk le, ahol x = 0 - 255 közötti érték, de vannak eltérések.
 ********************************************************************************/
const byte dim_curve[] = {
  0,   1,   1,   2,   2,   2,   2,   2,   2,   3,   3,   3,   3,   3,   3,   3,
  3,   3,   3,   3,   3,   3,   3,   4,   4,   4,   4,   4,   4,   4,   4,   4,
  4,   4,   4,   5,   5,   5,   5,   5,   5,   5,   5,   5,   5,   6,   6,   6,
  6,   6,   6,   6,   6,   7,   7,   7,   7,   7,   7,   7,   8,   8,   8,   8,
  8,   8,   9,   9,   9,   9,   9,   9,   10,  10,  10,  10,  10,  11,  11,  11,
  11,  11,  12,  12,  12,  12,  12,  13,  13,  13,  13,  14,  14,  14,  14,  15,
  15,  15,  16,  16,  16,  16,  17,  17,  17,  18,  18,  18,  19,  19,  19,  20,
  20,  20,  21,  21,  22,  22,  22,  23,  23,  24,  24,  25,  25,  25,  26,  26,
  27,  27,  28,  28,  29,  29,  30,  30,  31,  32,  32,  33,  33,  34,  35,  35,
  36,  36,  37,  38,  38,  39,  40,  40,  41,  42,  43,  43,  44,  45,  46,  47,
  48,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,
  63,  64,  65,  66,  68,  69,  70,  71,  73,  74,  75,  76,  78,  79,  81,  82,
  83,  85,  86,  88,  90,  91,  93,  94,  96,  98,  99,  101, 103, 105, 107, 109,
  110, 112, 114, 116, 118, 121, 123, 125, 127, 129, 132, 134, 136, 139, 141, 144,
  146, 149, 151, 154, 157, 159, 162, 165, 168, 171, 174, 177, 180, 183, 186, 190,
  193, 196, 200, 203, 207, 211, 214, 218, 222, 226, 230, 234, 238, 242, 248, 255,
};

// létrehozzuk az állandókat, az integer egész számot tárol:
#define led  10        // Idea van kötve a LED
#define potm A4        // Ide van kötve a potméter

void setup() {
  setup_7seg();        // Multifunkciós kártya inicializálás
}

void loop() {
  int reading = analogRead(potm);
  analogWrite(led, 255 - dim_curve[reading>>2]);
  // Várunk, hogy a szemünk is láthassa a változást:
  delay(50);
}

/**************************************************
 * A Multifunkciós kártya 7-szegmens kijelzőjének * 
 * inicializálása és elsötétítése                 *
 **************************************************/
#define LATCH_DIO 4
#define CLK_DIO   7
#define DATA_DIO  8
void setup_7seg(void) {
  /* Vezérlő lábak konfigurálása */
  pinMode(LATCH_DIO, OUTPUT);
  pinMode(CLK_DIO, OUTPUT);
  pinMode(DATA_DIO, OUTPUT);
  digitalWrite(LATCH_DIO, LOW);
  shiftOut(DATA_DIO, CLK_DIO, MSBFIRST, 0xFF);
  shiftOut(DATA_DIO, CLK_DIO, MSBFIRST, 0x00 );
  digitalWrite(LATCH_DIO, HIGH);
}
