/* servo_PWM16

   Szervo motor 16 bites PWM vezérlése potméterrel.
   A vezérlőjel a D9 kivezetésen jelenik meg.

   A 16 bites PWM Timer1 felhasználásával valósítható meg a D9 (OCR1A)
   illetve a D10 (OCR1B) kivezetéseken. Ebben a programban csak D9
   vezérlését implementáltuk.

   Timer1 előosztóját 1:8 leosztásra állítjuk, a 20 ms-os periódushoz
   így 40 000-ig kell számlálni és az 1000 - 2000 us kitöltéshez
   2000 - 4000 közötti számot kell ICR1-be írni.

   Némely gyártó teméke szélesebb tartományba eső jelet igényel,
   ebben az esetben próbálkozhatunk a
       OCR1A = sum / 64 + 1000;            //  4 * ADC + 1000
   beállítással, ami kb. 500 - 2500 us szélességú jelet produkál.

   Megjegyzés: Bekapcsoláskor egy közepes értékkel indítsunk, s óvatosan
   közelítsük meg a végkitéréseket!
*/

const uint16_t MAX_PWM_VALUE = 40000;  // Periódus = 20 ms

void setup() {
  pinMode(9, OUTPUT);
  TCCR1A = bit(COM1A1) |               // Non-inv PWM
           bit(WGM11);                 // Mode 14: Fast PWM, TOP=ICR1
  TCCR1B = bit(WGM13) | bit(WGM12) |
           bit(CS11);                  // Prescaler 1:8
  ICR1 = MAX_PWM_VALUE;                // TOP counter value
}

void loop() {
  uint32_t sum = 0;
  for (uint16_t i = 0; i < 256; i++)  {
    sum += analogRead(A4);
  }
  OCR1A = sum / 128 + 2000;            //  2 * ADC + 2000
  sum = 0;
  delay(40);
}
