/* two_servo_PWM16

   Két szervo motor 16 bites PWM vezérlése potméterekkel.
   A vezérlőjel a D9 és a D10 kivezetésen jelenik meg.

   A 16 bites PWM Timer1 felhasználásával valósítható meg, a
   kitöltés az OCR1A (D9), illetve az OCR1B (D10) regiszterek
   írásával, a periódus pedig az ICR1 regiszter írásával állítható be.

   Timer1 előosztóját 1:8 leosztásra állítjuk, a 20 ms-os periódushoz
   így 40 000-ig kell számlálni és az 1000 - 2000 us kitöltéshez
   2000 - 4000 közötti számot kell ICR1-be írni.

   Némely gyártó teméke szélesebb tartományba eső jelet igényel,
   ebben az esetben próbálkozhatunk 1000 - 5000 közötti számokkal.

   Megjegyzés: Bekapcsoláskor egy közepes értékkel indítsunk, s óvatosan
   közelítsük meg a végkitéréseket!

   Felhasznált forrás: T.K. Hareendran - Arduino & Advanced 16-bit PWM
   https://www.codrey.com/arduino-projects/arduino-advanced-16-bit-pwm/
*/

void setup() {
  setupPWM16();
}

void loop() {
  uint16_t duty = 0;
  duty = analogRead(A0) * 2 + 2000;
  analogWrite16(9, duty);
  delay(25);
  duty = analogRead(A1) * 2 + 2000;
  analogWrite16(10, duty);
  delay(25);
}

void setupPWM16() {
  DDRB |= bit(PB1) | bit(PB2);       // D9 & D10 output
  TCCR1A = bit(COM1A1) | bit(COM1B1) // Non-Inv PWM
           | bit(WGM11);             // Mode 14: Fast PWM, TOP=ICR1
  TCCR1B = bit(WGM13) | bit(WGM12)
           | bit(CS11);              // Prescaler 1:8
  ICR1 = 40000;                      // TOP counter value (Relieving OCR1A*)
}

void analogWrite16(uint8_t pin, uint16_t val) {
  switch (pin) {
    case 9: OCR1A = val; break;
    case 10: OCR1B = val; break;
  }
}
